@@ -4,10 +4,10 @@ use strict;
 use warnings;
 use 5.10.0;
 
-our $VERSION = '0.165';
+our $VERSION = '0.166';
 
 use Term::Choose::Constants qw( WIDTH_CURSOR );
-use Term::Choose::Screen    qw( clear_screen );
+use Term::Choose::Screen    qw( clear_screen clear_to_end_of_line );
 use Term::Choose::Util      qw( get_term_width );
 
 
@@ -15,26 +15,28 @@ sub new {
     my ( $class, $self ) = @_;
     bless $self, $class;
     $self->{show_progress_bar} //= 1;
-    if ( $self->{show_progress_bar} ) {
-        print clear_screen;
-        print "\rComputing: ";
-    }
     return $self;
 }
 
 
 sub set_progress_bar {
     my ( $self ) = @_;
-    my $term_w = get_term_width();
-    if ( $^O ne 'MSWin32' && $^O ne 'cygwin' ) {
-        $term_w += WIDTH_CURSOR;
-    }
+    my $term_w = get_term_width() + $Term::TablePrint::extra_w;
     $self->{fmt} = "\rComputing: %3d%% [%s]";
     $self->{short_print} = $term_w < 25 ? 1 : 0;
-    $self->{bar_w} = $term_w - length( sprintf $self->{fmt}, 100, '' ) + 1; # +1: lenght("\r") == 1
+    if ( $self->{short_print} ) {
+        $self->{bar_w} = $term_w
+    }
+    else {
+        $self->{bar_w} = $term_w - length( sprintf $self->{fmt}, 100, '' ) + 1; # +1: lenght("\r") == 1
+    }
     $self->{step} = int( $self->{total} / $self->{bar_w} || 1 );
     $self->{count} //= 0;
     $self->{next_update} ||= $self->{step};
+    if ( ! $self->{count} ) {
+        print clear_screen;
+        print "\rComputing: ";
+    }
     return;
 }
 
@@ -43,7 +45,8 @@ sub update_progress_bar {
     my ( $self ) = @_;
     my $multi = int( $self->{count} / ( $self->{total} / $self->{bar_w} ) ) || 1;
     if ( $self->{short_print} ) {
-        print "\r" . ( '=' x $multi ) . ( ' ' x $self->{bar_w} - $multi );
+        print "\r", clear_to_end_of_line;
+        print( ( '=' x $multi ) . ( ' ' x ( $self->{bar_w} - $multi ) ) );
     }
     else {
         printf $self->{fmt}, ( $self->{count} / $self->{total} * 100 ), ( '=' x $multi ) . ( ' ' x ( $self->{bar_w} - $multi ) );
@@ -65,7 +68,7 @@ Term::TablePrint::ProgressBar - Show a progress bar.
 
 =head1 VERSION
 
-Version 0.165
+Version 0.166
 
 =cut
 
